package com.sybase.sup.samples.objectapi;

import java.util.Hashtable;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.KeywordFilterField;
import SUP101.Customer;
import SUP101.SUP101DB;

import com.sybase.collections.ObjectList;
import com.sybase.persistence.ChangeLog;
import com.sybase.persistence.DefaultCallbackHandler;
import com.sybase.persistence.Query;
import com.sybase.persistence.SynchronizationAction;
import com.sybase.persistence.SynchronizationContext;
import com.sybase.persistence.SynchronizationStatus;

/**
 * The database class callback handler implementation.
 */
public class CustomerDBCallback extends DefaultCallbackHandler
{
    /**
     * A map tracks all customer's surrogate keys which are changed by server
     * If a customer is reloaded from database, it will be removed from this map.
     */
    private Hashtable _changedSKs = new Hashtable();
    
    private CustomerSampleScreen _screen;
    
    private static CustomerDBCallback _instance = new CustomerDBCallback();
    
    private CustomerDBCallback()
    {
    }
    
    public static CustomerDBCallback getInstace()
    {
        return _instance;
    }
    
    /**
     * Setter of UI CustomerSampleScreen
     * Callback needs to call it to refresh UI
     */
    public void setCustomerSampleScreen(CustomerSampleScreen screen)
    {
        _screen = screen;
    }

    /**
     * Inside onSynchronize, we'll check if anything has been changed on server.
     * If changed, we'll save changed surrogate key inside a map.
     * Later on, when refreshing UI, we can see if data needs to be reloaded by querying the map.
     */
    public int onSynchronize(ObjectList groups, SynchronizationContext context)
    {
        int progress = 0;
        String status = "";
        if(context.getStatus() == SynchronizationStatus.STARTING)
        {
            progress = 0;
            status = "Synchronizing ... ";
        }
        else if(context.getStatus() == SynchronizationStatus.FINISHING || context.getStatus() == SynchronizationStatus.ASYNC_REPLAY_UPLOADED)
        {
            progress = 100;
            status = "Synchronization Done";
        }
        
        if(_screen != null)
        {
            _screen.notifySynchronizeProgress(progress, status);
            
            if(context.getStatus() == SynchronizationStatus.FINISHING || context.getStatus() == SynchronizationStatus.ASYNC_REPLAY_UPLOADED)
            {
                ObjectList changeLogs = SUP101DB.getChangeLogs(new Query());
            
                int sz = changeLogs.size();
                for(int i = 0; i < sz; i++)
                {
                    ChangeLog changeLog = (ChangeLog)changeLogs.item(i);
                    long sk = changeLog.getSurrogateKey();
                
                    _changedSKs.put(new Long(sk), Boolean.TRUE);
                }
            }
        }

        SUP101DB.deleteChangeLogs();
        return SynchronizationAction.CONTINUE;
    }
    
    /**
     * Return customer from database if the customer is changed by Server.
     * @param c target customer
     * @return the reloaded customer
     */
    public Customer reLoadIfNeeded(Customer c)
    {
        Long sk = new Long(c.getSurrogateKey());
        if(_changedSKs.containsKey(sk))
        {
            c.refresh();
            _changedSKs.remove(sk);
        }
        return c;
    }
}
