package com.sybase.sup.samples.objectapi;
/*
 * Created on Jul 20, 2010
 *
 * Copyright (c) Sybase, Inc. 2010   
 * All rights reserved.                                    
 */

import net.rim.device.api.collection.util.SortedReadableList;
import net.rim.device.api.ui.component.KeywordProvider;
import net.rim.device.api.util.Comparator;
import net.rim.device.api.util.StringUtilities;
import SUP101.Customer;

/**
 * Instances of this SortedReadableList class will contain a list of customers
 * derived from a vector of Customer objects passed into the constructor. The
 * class is also a KeywordProvider implementation.
 */
public class CustomerList extends SortedReadableList
        implements KeywordProvider
{
    /**
     * Creates a customer list based on a Vector of customers.
     */
    public CustomerList()
    {
        super(new CustomerListComparator());
    }

    /**
     * @see net.rim.device.api.ui.component.KeywordProvider#getKeywords(Object
     *      element)
     */
    public String[] getKeywords(Object element)
    {
        if ( element instanceof Customer )
        {
            Customer customer = (Customer) element;
            return StringUtilities.stringToWords(customer.getFname() + " " + customer.getLname());
        }
        return null;
    }

    /**
     * A Comparator class used for sorting our Customer objects by name.
     */
    final static class CustomerListComparator
            implements Comparator
    {
        /**
         * Compares two customers by comparing their names' in alphabetical
         * order.
         * 
         * @see net.rim.device.api.util.Comparator#compare(Object, Object)
         */
        public int compare(Object o1, Object o2)
        {
            if ( o1 == null || o2 == null ) throw new IllegalArgumentException("Cannot compare null customers");
            return ((Customer)o1).getFname().compareTo(((Customer)o2).getFname());
        }
    }
}
