package com.sybase.sup.samples.objectapi;
/*
 * Created on Jul 20, 2010
 *
 * Copyright (c) Sybase, Inc. 2010   
 * All rights reserved.                                    
 */


import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.KeywordFilterField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import SUP101.Customer;
import SUP101.SUP101DB;

import com.sybase.collections.ObjectList;

/**
 * @author bdeng
 *
 * This class represents the main screen for the Customer Sample application.
 * 
 * Note: For the sake of simplicity, this sample application may not leverage
 * resource bundles and resource strings.  However, it is STRONGLY recommended
 * that application developers make use of the localization features available
 * within the BlackBerry development platform to ensure a seamless application
 * experience across a variety of languages and geographies.  For more information
 * on localizing your application, please refer to the BlackBerry Java Development
 * Environment Development Guide.
 */
public final class CustomerSampleScreen extends MainScreen
{
    private CustomerSample     _app;
    private KeywordFilterField _keywordFilterField;

    // synchronization indicator
    private GaugeField         _syncProgressField;

    /**
     * Creates a new CustomerSampleScreen.
     * 
     * @param app The UiApplication creating an instance of this class.
     */
    public CustomerSampleScreen(CustomerSample app)
    {
        // A reference to the UiApplication instance for use in this class.
        _app = app;

        // We need a reference to the UiApplication's KeywordFilterField.
        _keywordFilterField = _app.getKeywordFilterField();        
        initialize();
    }

    /**
     * Intercepts the ENTER key and displays info screen.
     * 
     * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
     */
    protected boolean keyChar(char key, int status, int time)
    {
        if ( key == Characters.ENTER )
        {
            displayInfoScreen();
            return true; // We've consumed the event.
        }
        return super.keyChar(key, status, time);
    }

    /**
     * Handles a trackball click.
     * 
     * @see net.rim.device.api.ui.Screen#invokeAction(int)
     */
    public boolean invokeAction(int action)
    {
        switch (action)
        {
            case ACTION_INVOKE: // Trackball click.
                displayInfoScreen();
                return true; // We've consumed the event.
        }
        return super.invokeAction(action);
    }

    /**
     * Creates an InfoScreen instance and pushes it onto the stack for
     * rendering.
     */
    private void displayInfoScreen()
    {
        // Retrieve the selected Customer and use it to invoke a new InfoScreen.
        Customer customer = (Customer) _keywordFilterField.getSelectedElement();
        if ( customer != null )
        {
            InfoScreen infoScreen = new InfoScreen(this, customer);
            _app.pushScreen(infoScreen);
        }
    }

    /**
     * Prevent the save dialog from being displayed.
     * 
     * @see net.rim.device.api.ui.container.MainScreen#onSavePrompt()
     */
    public boolean onSavePrompt()
    {
        return true;
    }

    // Inner classes------------------------------------------------------------
    /**
     * A MainScreen class to display/update secondary information for a selected
     * customer.
     */
    private final class InfoScreen extends MainScreen
    {
        private Customer             _customer;

        private BasicEditField       _fnameField;
        private BasicEditField       _lnameField;
        private BasicEditField       _companyField;
        private BasicEditField       _addressField;
        private BasicEditField       _stateField;
        private BasicEditField       _cityField;
        private BasicEditField       _phoneField;
        private BasicEditField       _zipField;

        /**
         * Constructs a screen to display
         * 
         * @param customer
         *            The customer to display secondary information about
         */
        InfoScreen(CustomerSampleScreen sampleScreen, Customer customer)
        {
            _customer = customer;

            // Set up and display UI elements.
            setTitle("Update Customer");
            _fnameField = new BasicEditField("First name: ", customer.getFname(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _lnameField = new BasicEditField("Last name: ", customer.getLname(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _companyField = new BasicEditField("Company: ", customer.getCompany_name(),
                    BasicEditField.DEFAULT_MAXCHARS, Field.FOCUSABLE);
            _addressField = new BasicEditField("Address: ", customer.getAddress(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _stateField = new BasicEditField("State: ", customer.getState(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _cityField = new BasicEditField("City: ", customer.getCity(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _phoneField = new BasicEditField("Phone: ", customer.getPhone(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _zipField = new BasicEditField("Zip: ", customer.getZip(), BasicEditField.DEFAULT_MAXCHARS, Field.FOCUSABLE);
            add(_fnameField);
            add(_lnameField);
            add(_companyField);
            add(_addressField);
            add(_stateField);
            add(_cityField);
            add(_phoneField);
            add(_zipField);

            HorizontalFieldManager hfm = new HorizontalFieldManager(Field.FIELD_HCENTER);

            ButtonField submitButton = new ButtonField("submit", Field.FIELD_RIGHT)
            {
                protected boolean navigationClick(int status, int time)
                {
                    submit();
                    return true;
                }
            };

            ButtonField cancelButton = new ButtonField("cancel", Field.FIELD_LEFT)
            {
                protected boolean navigationClick(int status, int time)
                {
                    close();
                    return true;
                }
            };

            hfm.add(submitButton);
            hfm.add(cancelButton);

            add(hfm);
        }

        protected void submit()
        {
            _customer.setFname(_fnameField.getText().trim());
            _customer.setLname(_lnameField.getText().trim());
            _customer.setCompany_name(_companyField.getText().trim());
            _customer.setAddress(_addressField.getText().trim());
            _customer.setState(_stateField.getText().trim());
            _customer.setCity(_cityField.getText().trim());
            _customer.setPhone(_phoneField.getText().trim());
            _customer.setZip(_zipField.getText().trim());
            _customer.save();
            _customer.submitPending();
            close();
            CustomerSampleScreen.this.synchronize();
        }
    }

    private void initialize()
    {
        _syncProgressField = new GaugeField("Initializing ...", 0, 100, 0, GaugeField.NO_TEXT | GaugeField.LABEL_AS_PROGRESS)
        {
            protected void paint(Graphics g)
            {
                super.paint(g);
                int indicatorHeigth = 20;
                int fontSize = 12;
                int y = getHeight() - indicatorHeigth;

                // setting up font style
                Font f = Font.getDefault().derive(Font.PLAIN, fontSize);
                g.setFont(f);
                g.setColor(0x00000000);
                g.setDrawingStyle(Graphics.DRAWSTYLE_AALINES, true);
                // paint the synchronizing indicator as it can't be
                // displayed in the progress bar using
                // LABEL_AS_PROGRESS style
                g.drawText(getLabel(), 1, y + (indicatorHeigth - fontSize) / 2);
                g.setColor(0x00BEBEBE);
                g.drawRect(0, 0, getWidth(), getHeight());
            }
        };
        
        setStatus(_syncProgressField);
        
        new Thread(
                new Runnable()
                {
                    public void run()
                    {
                        ObjectAPIUtil.initialize();
                        
                        UiApplication.getUiApplication().invokeLater(new Runnable()
                        {
                            public void run()
                            {
                                _app.refreshData();                                
                                _syncProgressField.setValue(100);
                                _syncProgressField.setLabel("Initialize succeeds");      
                                CustomerDBCallback.getInstace().setCustomerSampleScreen(CustomerSampleScreen.this);
                            }
                        });
                    }
                }
            ).start();
    }
    
    public void notifySynchronizeProgress(int progress, String status)
    {
        if(_syncProgressField != null)
        {
            final int fProgress = progress;
            final String fstatus = status;
            UiApplication.getUiApplication().invokeLater(new Runnable()
            {
                public void run()
                {
                    _syncProgressField.setValue(fProgress);
                    _syncProgressField.setLabel(fstatus);
                    _keywordFilterField.invalidate();
                }
            });
        }
    }
    
    // synchronizes the mbo
    private void synchronize()
    {
       try
       {           
           _syncProgressField = new GaugeField("", 0, 100, 0, GaugeField.NO_TEXT | GaugeField.LABEL_AS_PROGRESS)
           {
               protected void paint(Graphics g)
               {
                   super.paint(g);
                   int indicatorHeigth = 20;
                   int fontSize = 12;
                   int y = getHeight() - indicatorHeigth;

                   // setting up font style
                   Font f = Font.getDefault().derive(Font.PLAIN, fontSize);
                   g.setFont(f);
                   g.setColor(0x00000000);
                   g.setDrawingStyle(Graphics.DRAWSTYLE_AALINES, true);
                   // paint the synchronizing indicator as it can't be
                   // displayed in the progress bar using
                   // LABEL_AS_PROGRESS style
                   g.drawText(getLabel(), 1, y + (indicatorHeigth - fontSize) / 2);
                   g.setColor(0x00BEBEBE);
                   g.drawRect(0, 0, getWidth(), getHeight());
               }
           };

           setStatus(_syncProgressField);
           ObjectAPIUtil.beginSynchronize();
       }
       catch (Throwable t)
       {
           t.printStackTrace();
       }
    }
}
